@@ -249,8 +249,8 @@
 	succeeds)
     (luna-call-next-method)))
 
-(luna-define-method elmo-folder-delete-messages ((folder elmo-localdir-folder)
-						 numbers)
+(luna-define-method elmo-folder-delete-messages-internal
+  ((folder elmo-localdir-folder) numbers)
   (dolist (number numbers)
     (elmo-localdir-delete-message folder number))
   t)
@@ -323,13 +323,6 @@
     (rename-file old new)
     t))
 
-(defsubst elmo-localdir-field-condition-match (folder condition
-						      number number-list)
-  (elmo-file-field-condition-match
-   (expand-file-name (int-to-string number)
-		     (elmo-localdir-folder-directory-internal folder))
-   condition number number-list))
-
 (luna-define-method elmo-folder-pack-numbers ((folder elmo-localdir-folder))
   (let* ((dir (elmo-localdir-folder-directory-internal folder))
 	 (msgdb (elmo-folder-msgdb folder))
@@ -355,6 +348,7 @@
 	  (elmo-message-entity-set-number entity new-number))
 	(elmo-msgdb-append-entity new-msgdb entity
 				  (elmo-msgdb-flags msgdb old-number))
+	(elmo-emit-signal 'message-number-changed folder old-number new-number)
 	(setq new-number (1+ new-number))))
     (message "Packing...done")
     (elmo-folder-set-msgdb-internal folder new-msgdb)))
@@ -362,12 +356,6 @@
 (luna-define-method elmo-folder-message-file-p ((folder elmo-localdir-folder))
   t)
 
-(luna-define-method elmo-message-file-name ((folder elmo-localdir-folder)
-					    number)
-  (expand-file-name
-   (int-to-string number)
-   (elmo-localdir-folder-directory-internal folder)))
-
 (defun elmo-localdir-locked-p ()
   (if elmo-localdir-lockfile-list
       (let ((lock elmo-localdir-lockfile-list))
