@@ -61,6 +61,11 @@ set as non-nil.")
   :type 'boolean
   :group 'elmo)
 
+(defconst elmo-pop3-folder-name-syntax `(([user ".+"])
+					 (?/ [auth ".+"])
+					 (?: [uidl "^[A-Za-z]+$"])
+					 ,@elmo-net-folder-name-syntax))
+
 (defvar sasl-mechanism-alist)
 
 (defvar elmo-pop3-total-size nil)
@@ -93,38 +98,34 @@ Debug information is inserted in the buffer \"*POP3 DEBUG*\"")
 	     (append elmo-pop3-stream-type-alist
 		     elmo-network-stream-type-alist)
 	   elmo-network-stream-type-alist))
-	parse)
+	tokens auth uidl)
+    (setq tokens (car (elmo-parse-separated-tokens
+		       name
+		       elmo-pop3-folder-name-syntax)))
     ;; user
-    (setq parse (elmo-parse-token name "/@:!"))
     (elmo-net-folder-set-user-internal folder
-				       (if (eq (length (car parse)) 0)
-					   elmo-pop3-default-user
-					 (car parse)))
+				       (or (cdr (assq 'user tokens))
+					   elmo-pop3-default-user))
     ;; auth
-    (setq parse (elmo-parse-prefixed-element ?/ (cdr parse) ":@!"))
+    (setq auth (cdr (assq 'auth tokens)))
     (elmo-net-folder-set-auth-internal folder
-				       (if (eq (length (car parse)) 0)
-					   elmo-pop3-default-authenticate-type
-					 (intern (downcase (car parse)))))
+				       (if auth
+					   (intern (downcase auth))
+					 elmo-pop3-default-authenticate-type))
     ;; uidl
-    (setq parse (elmo-parse-prefixed-element ?: (cdr parse) "@:!" "^[a-z]+"))
+    (setq uidl (cdr (assq 'uidl tokens)))
     (elmo-pop3-folder-set-use-uidl-internal folder
-					    (if (eq (length (car parse)) 0)
-						elmo-pop3-default-use-uidl
-					      (string= (car parse) "uidl")))
+					    (if uidl
+						(string= uidl "uidl")
+					      elmo-pop3-default-use-uidl))
     ;; network
-    (elmo-net-parse-network folder (cdr parse))
-    (unless (elmo-net-folder-server-internal folder)
-      (elmo-net-folder-set-server-internal folder
-					   elmo-pop3-default-server))
-    (unless (elmo-net-folder-port-internal folder)
-      (elmo-net-folder-set-port-internal folder
-					 elmo-pop3-default-port))
-    (unless (elmo-net-folder-stream-type-internal folder)
-      (elmo-net-folder-set-stream-type-internal
-       folder
-       (elmo-get-network-stream-type
-	elmo-pop3-default-stream-type)))
+    (elmo-net-folder-set-parameters
+     folder
+     tokens
+     (list :server	elmo-pop3-default-server
+	   :port	elmo-pop3-default-port
+	   :stream-type
+	   (elmo-get-network-stream-type elmo-pop3-default-stream-type)))
     folder))
 
 ;;; POP3 session
@@ -283,7 +284,6 @@ CODE is one of the following:
      nil 'no-log)
     (setq response (elmo-pop3-read-response process t))
     (unless (eq (car response) 'ok)
-      (delete-process process)
       (signal 'elmo-open-error '(elmo-pop-auth-user)))
     (elmo-pop3-send-command  process
 			     (format
@@ -292,53 +292,44 @@ CODE is one of the following:
 			       (elmo-network-session-password-key session)))
 			     nil 'no-log)
     (setq response (elmo-pop3-read-response process t))
-    (unless (eq (car response) 'ok)
-      (delete-process process)
-      (unless (eq 'ok (car response))
-	(if (or (eq (car response) 'in-use)
-		(eq (car response) 'login-delay))
-	    (error (cond ((eq (car response) 'in-use)
-			  "Maildrop is currently in use")
-			 ((eq (car response) 'login-delay)
-			  "Not allowed to login \
-until the login delay period has expired")))
-	  (signal 'elmo-authenticate-error
-		  '(elmo-pop-auth-user)))))
+    (case (car response)
+      (ok)
+      (in-use
+       (error "Maildrop is currently in use"))
+      (login-delay
+       (error "Not allowed to login until the login delay period has expired"))
+      (t
+       (signal 'elmo-authenticate-error '(elmo-pop-auth-user))))
     (car response)))
 
 (defun elmo-pop3-auth-apop (session)
-  (if (string-match "^\+OK .*\\(<[^\>]+>\\)"
-		    (elmo-network-session-greeting-internal session))
-      ;; good, APOP ready server
-      (progn
-	(elmo-pop3-send-command
-	 (elmo-network-session-process-internal session)
-	 (format "apop %s %s"
-		 (elmo-network-session-user-internal session)
-		 (md5
-		  (concat (match-string
-			   1
-			   (elmo-network-session-greeting-internal session))
-			  (elmo-get-passwd
-			   (elmo-network-session-password-key session)))))
-	 nil 'no-log)
-	(let ((response (elmo-pop3-read-response
-			 (elmo-network-session-process-internal session)
-			 t)))
-	  (unless (eq (car response) 'ok)
-	    (delete-process (elmo-network-session-process-internal session))
-	    (unless (eq 'ok (car response))
-	      (if (or (eq (car response) 'in-use)
-		      (eq (car response) 'login-delay))
-		  (error (cond ((eq (car response) 'in-use)
-				"Maildrop is currently in use")
-			       ((eq (car response) 'login-delay)
-				"Not allowed to login \
-until the login delay period has expired")))
-		(signal 'elmo-authenticate-error
-			'(elmo-pop-auth-apop)))))
-	  (car response)))
-    (signal 'elmo-open-error '(elmo-pop3-auth-apop))))
+  (unless (string-match "^\+OK .*\\(<[^\>]+>\\)"
+			(elmo-network-session-greeting-internal session))
+    (signal 'elmo-open-error '(elmo-pop3-auth-apop)))
+  ;; good, APOP ready server
+  (elmo-pop3-send-command
+   (elmo-network-session-process-internal session)
+   (format "apop %s %s"
+	   (elmo-network-session-user-internal session)
+	   (md5
+	    (concat (match-string
+		     1
+		     (elmo-network-session-greeting-internal session))
+		    (elmo-get-passwd
+		     (elmo-network-session-password-key session)))))
+   nil 'no-log)
+  (let ((response (elmo-pop3-read-response
+		   (elmo-network-session-process-internal session)
+		   t)))
+    (case (car response)
+      (ok)
+      (in-use
+       (error "Maildrop is currently in use"))
+      (login-delay
+       (error "Not allowed to login until the login delay period has expired"))
+      (t
+       (signal 'elmo-authenticate-error '(elmo-pop-auth-apop))))
+    (car response)))
 
 (luna-define-method elmo-network-initialize-session-buffer :after
   ((session elmo-pop3-session) buffer)
@@ -378,16 +369,15 @@ until the login delay period has expired")))
 			(elmo-network-session-process-internal session))
     (let* ((process (elmo-network-session-process-internal session))
 	   (auth (elmo-network-session-auth-internal session))
-	   (auth (mapcar '(lambda (mechanism) (upcase (symbol-name mechanism)))
-			 (if (listp auth) auth (list auth))))
-	   sasl-mechanisms
-	   client name step response mechanism
-	   sasl-read-passphrase)
+	   (auth (mapcar (lambda (mechanism) (upcase (symbol-name mechanism)))
+			 (if (listp auth) auth (list auth)))))
       (or (and (string= "USER" (car auth))
 	       (elmo-pop3-auth-user session))
 	  (and (string= "APOP" (car auth))
 	       (elmo-pop3-auth-apop session))
-	  (progn
+	  (let (sasl-mechanisms
+		client name step response mechanism
+		sasl-read-passphrase)
 	    (require 'sasl)
 	    (setq sasl-mechanisms (mapcar 'car sasl-mechanism-alist))
 	    (setq mechanism (sasl-find-mechanism auth))
@@ -405,10 +395,9 @@ until the login delay period has expired")))
 	    (elmo-network-session-set-auth-internal session
 						    (intern (downcase name)))
 	    (setq sasl-read-passphrase
-		  (function
-		   (lambda (prompt)
-		     (elmo-get-passwd
-		      (elmo-network-session-password-key session)))))
+		  (lambda (prompt)
+		    (elmo-get-passwd
+		     (elmo-network-session-password-key session))))
 	    (setq step (sasl-next-step client nil))
 	    (elmo-pop3-send-command
 	     process
@@ -422,17 +411,17 @@ until the login delay period has expired")))
 	    (catch 'done
 	      (while t
 		(setq response (elmo-pop3-read-response process t))
-		(unless (eq 'ok (car response))
-		  (if (or (eq (car response) 'in-use)
-			  (eq (car response) 'login-delay))
-		      (error (cond ((eq (car response) 'in-use)
-				    "Maildrop is currently in use")
-				   ((eq (car response) 'login-delay)
-				    "Not allowed to login \
-until the login delay period has expired")))
-		    (signal 'elmo-authenticate-error
-			    (list (intern (concat "elmo-pop3-auth-"
-						  (downcase name)))))))
+		(case (car response)
+		  (ok)
+		  (in-use
+		   (error "Maildrop is currently in use"))
+		  (login-delay
+		   (error "Not allowed to login \
+until the login delay period has expired"))
+		  (t
+		   (signal 'elmo-authenticate-error
+			   (list (intern (concat "elmo-pop3-auth-"
+						 (downcase name)))))))
 		(if (sasl-next-step client step)
 		    ;; Bogus server?
 		    (signal 'elmo-authenticate-error
