@@ -530,18 +530,17 @@ Return value is a cons cell of (STRUCTURE . REST)"
 (defvar elmo-passwd-alist nil)
 
 (defun elmo-passwd-alist-load ()
-  (with-temp-buffer
-    (let ((filename (expand-file-name elmo-passwd-alist-file-name
-				      elmo-msgdb-directory))
-	  insert-file-contents-pre-hook	; To avoid autoconv-xmas...
-	  insert-file-contents-post-hook
-	  ret-val)
-      (if (not (file-readable-p filename))
-	  ()
-	(insert-file-contents filename)
-	(condition-case nil
-	    (read (current-buffer))
-	  (error nil nil))))))
+  (let ((filename (expand-file-name elmo-passwd-alist-file-name
+				    elmo-msgdb-directory)))
+    (if (not (file-readable-p filename))
+	()
+      (with-temp-buffer
+	(let (insert-file-contents-pre-hook ; To avoid autoconv-xmas...
+	      insert-file-contents-post-hook)
+	  (insert-file-contents filename)
+	  (goto-char (point-min))
+	  (ignore-errors
+	   (read (current-buffer))))))))
 
 (defun elmo-passwd-alist-clear ()
   "Clear password cache."
@@ -882,8 +881,12 @@ the directory becomes empty after deletion."
 	  ))))
 
 (defun elmo-list-filter (l1 l2)
-  "Rerurn a list from L2 in which each element is a member of L1."
-  (elmo-delete-if (lambda (x) (not (memq x l1))) l2))
+  "Return a list from L2 in which each element is a member of L1."
+  (let (result)
+    (dolist (element l2)
+      (if (memq element l1)
+	(setq result (cons element result))))
+    (nreverse result)))
 
 (defsubst elmo-list-delete-if-smaller (list number)
   (let ((ret-val (copy-sequence list)))
@@ -1003,7 +1006,9 @@ Emacs 19.28 or earlier does not have `unintern'."
   (and string
        (elmo-with-enable-multibyte
 	 (encode-mime-charset-string
-	  (eword-decode-and-unfold-unstructured-field-body string)
+	  (or (ignore-errors
+	       (eword-decode-and-unfold-unstructured-field-body string))
+	      string)
 	  elmo-mime-charset))))
 
 (defsubst elmo-collect-field (beg end downcase-field-name)
@@ -1280,8 +1285,10 @@ SPEC is a list as followed (LABEL MAX-VALUE [FORMAT])."
 (defun elmo-decoded-field-body (field-name &optional mode)
   (let ((field-body (elmo-field-body field-name)))
     (and field-body
-	 (elmo-with-enable-multibyte
-	   (mime-decode-field-body field-body field-name mode)))))
+	 (or (ignore-errors
+	      (elmo-with-enable-multibyte
+		(mime-decode-field-body field-body field-name mode)))
+	     field-body))))
 
 (defun elmo-address-quote-specials (word)
   "Make quoted string of WORD if needed."
@@ -1299,12 +1306,11 @@ SPEC is a list as followed (LABEL MAX-VALUE [FORMAT])."
 
 (defun elmo-flatten (list-of-list)
   "Flatten LIST-OF-LIST."
-  (unless (null list-of-list)
-    (append (if (and (car list-of-list)
-		     (listp (car list-of-list)))
-		(car list-of-list)
-	      (list (car list-of-list)))
-	    (elmo-flatten (cdr list-of-list)))))
+  (if list-of-list
+      (apply #'nconc
+	     (mapcar (lambda (element)
+		       (if (consp element) element (list element)))
+		     list-of-list))))
 
 (defun elmo-y-or-n-p (prompt &optional auto default)
   "Same as `y-or-n-p'.
@@ -1315,11 +1321,14 @@ But if optional argument AUTO is non-nil, DEFAULT is returned."
 
 (defun elmo-string-member (string slist)
   (catch 'found
-    (while slist
-      (if (and (stringp (car slist))
-	       (string= string (car slist)))
-	  (throw 'found t))
-      (setq slist (cdr slist)))))
+    (dolist (element slist)
+      (cond ((null element))
+	    ((stringp element)
+	     (when (string= string element)
+	       (throw 'found t)))
+	    ((symbolp element)
+	     (when (string= string (symbol-value element))
+	       (throw 'found t)))))))
 
 (static-cond ((fboundp 'member-ignore-case)
        (defalias 'elmo-string-member-ignore-case 'member-ignore-case))
@@ -1525,21 +1534,19 @@ ELT must be a string.  Upper-case and lower-case letters are treated as equal."
       (and result
 	   (char-list-to-string (elmo-uniq-list result #'delq))))))
 
-(defun elmo-collect-separators-internal (specs)
-  (let (separators)
-    (while specs
-      (let ((spec (car specs)))
-	(cond
-	 ((listp spec)
-	  (setq separators (nconc (elmo-collect-separators-internal spec)
-				  separators)
-		specs (cdr specs)))
-	 ((characterp spec)
-	  (setq separators (cons spec separators)
-		specs nil))
-	 (t
-	  (setq specs nil)))))
-    separators))
+(defun elmo-collect-separators-internal (specs &optional separators)
+  (while specs
+    (let ((spec (car specs)))
+      (cond
+       ((listp spec)
+	(setq separators (elmo-collect-separators-internal spec separators)
+	      specs (cdr specs)))
+       ((characterp spec)
+	(setq separators (cons spec separators)
+	      specs nil))
+       (t
+	(setq specs nil)))))
+  separators)
 
 (defun elmo-collect-trail-separators (element specs)
   (cond
@@ -1800,6 +1807,18 @@ NUMBER-SET is altered."
 	(cdr result))
     (funcall function object)))
 
+(defun elmo-map-until-success (function sequence)
+  (let (result)
+    (while (and (null result)
+		sequence)
+      (setq result (funcall function (car sequence))
+	    sequence (cdr sequence)))
+    result))
+
+(defun elmo-string-match-substring (regexp string &optional matchn)
+  (when (string-match regexp string)
+    (match-string (or matchn 1) string)))
+
 (defun elmo-parse (string regexp &optional matchn)
   (or matchn (setq matchn 1))
   (let (list)
@@ -1809,6 +1828,26 @@ NUMBER-SET is altered."
 				  (match-end matchn)) list)))
     (nreverse list)))
 
+(defun elmo-find-list-match-value (specs getter)
+  (lexical-let ((getter getter))
+    (elmo-map-until-success
+     (lambda (spec)
+       (cond
+	((symbolp spec)
+	 (funcall getter spec))
+	((consp spec)
+	 (lexical-let ((value (funcall getter (car spec))))
+	   (when value
+	     (elmo-map-until-success
+	      (lambda (rule)
+		(cond
+		 ((stringp rule)
+		  (elmo-string-match-substring rule value))
+		 ((consp rule)
+		  (elmo-string-match-substring (car rule) value (cdr rule)))))
+	      (cdr spec)))))))
+     specs)))
+
 ;;; File cache.
 (defmacro elmo-make-file-cache (path status)
   "PATH is the cache file name.
