@@ -34,11 +34,6 @@
 (require 'elmo)
 (require 'wl-vars)
 
-(eval-when-compile
-  (defalias-maybe 'pgg-decrypt-region 'ignore)
-  (defalias-maybe 'pgg-display-output-buffer 'ignore)
-  (defalias-maybe 'pgg-verify-region 'ignore))
-
 ;;; Draft
 
 (defalias 'wl-draft-editor-mode 'mime-edit-mode)
@@ -224,21 +219,24 @@ It calls following-method selected from variable
       (kill-buffer buffer))))
 
 (defun wl-draft-attribute-recipients ()
-  (concat (mapconcat
-	   'identity
-	   (wl-draft-deduce-address-list
-	    (current-buffer)
-	    (point-min)
-	    (save-excursion
-	      (goto-char (point-min))
-	      (re-search-forward
-	       (concat
-		"^"
-		(regexp-quote mail-header-separator)
-		"$")
-	       nil t)
-	      (point)))
-	   ", ")))
+  (let (wl-draft-remove-group-list-contents)
+    (mapconcat
+     'identity
+     (nconc
+      (wl-draft-deduce-address-list
+       (current-buffer)
+       (point-min)
+       (save-excursion
+	 (goto-char (point-min))
+	 (re-search-forward
+	  (concat
+	   "^"
+	   (regexp-quote mail-header-separator)
+	   "$")
+	  nil t)
+	 (point)))
+      (wl-draft-parse-mailbox-list wl-draft-mime-bcc-field-name))
+     ", ")))
 
 (defun wl-draft-attribute-envelope-from ()
   (or wl-envelope-from
@@ -280,7 +278,7 @@ It calls following-method selected from variable
 
 (defun wl-draft-attribute-nntp-posting-port ()
   (wl-draft-nntp-attribute
-   'point
+   'port
    '(wl-nntp-posting-port elmo-nntp-default-port)))
 
 (defun wl-draft-attribute-value (attr)
@@ -477,41 +475,111 @@ It calls following-method selected from variable
 	  (setq wl-message-buffer nil)
 	  (wl-summary-sync nil "update"))))))
 
+;; PGP
+(static-cond
+ ((require 'epg nil t)
+  (defun wl-mime-pgp-decrypt-region (beg end &optional no-decode)
+    (require 'epg)
+    (message "Decrypting...")
+    (insert (prog1
+		(decode-coding-string
+		 (epg-decrypt-string
+		  (epg-make-context)
+		  (buffer-substring beg end))
+		 (if no-decode 'raw-text wl-cs-autoconv))
+	      (delete-region beg end)))
+    (message "Decrypting...done")
+    last-coding-system-used)
+
+  (defun wl-mime-pgp-verify-region (beg end &optional coding-system)
+    (require 'epg)
+    (let ((context (epg-make-context)))
+      (message "Verifying...")
+      (epg-verify-string
+       context
+       (encode-coding-string
+	(buffer-substring beg end)
+	(if coding-system
+	    (coding-system-change-eol-conversion coding-system 'dos)
+	  'raw-text-dos)))
+      (message "Verifying...done")
+      (when (epg-context-result-for context 'verify)
+	(epa-display-verify-result
+	 (epg-context-result-for context 'verify))))))
+
+ ((require 'pgg nil t)
+  (defun wl-mime-pgp-decrypt-region (beg end &optional no-decode)
+    (require 'pgg)
+    (let ((buffer-file-coding-system wl-cs-autoconv)
+	  status)
+      (setq status (pgg-decrypt-region beg end))
+      (if no-decode
+	  (when status
+	    (delete-region beg end)
+	    (insert-buffer-substring pgg-output-buffer))
+	(pgg-display-output-buffer beg end status))
+      (unless status
+	(error "Decryption is failed"))
+      last-coding-system-used))
+
+  (defun wl-mime-pgp-verify-region (beg end &optional coding-system)
+    (require 'pgg)
+    (let ((message-buffer (current-buffer))
+	  success)
+      (with-temp-buffer
+	(insert-buffer-substring message-buffer beg end)
+	(when coding-system
+	  (encode-coding-region (point-min) (point-max) coding-system))
+	(setq success (pgg-verify-region (point-min) (point-max) nil 'fetch)))
+      (mime-show-echo-buffer)
+      (set-buffer mime-echo-buffer-name)
+      (set-window-start
+       (get-buffer-window mime-echo-buffer-name)
+       (point-max))
+      (insert-buffer-substring
+       (if success
+	   pgg-output-buffer
+	 pgg-errors-buffer)))))
+ (t
+  (defun wl-mime-pgp-decrypt-region (beg end &optional no-decode)
+    (error "Does not support PGP decryption"))
+
+  (defun wl-mime-pgp-verify-region (beg end &optional coding-system)
+    (error "Does not support PGP verification"))))
+
 (defun wl-message-decrypt-pgp-nonmime ()
   "Decrypt PGP encrypted region"
   (interactive)
-  (require 'pgg)
   (save-excursion
     (beginning-of-line)
-    (if (or (re-search-forward "^-+END PGP MESSAGE-+$" nil t)
-	    (re-search-backward "^-+END PGP MESSAGE-+$" nil t))
-	(let (beg end status)
-	  (setq end (match-end 0))
-	  (if (setq beg (re-search-backward "^-+BEGIN PGP MESSAGE-+$" nil t))
-	      (let ((inhibit-read-only t)
-		    (buffer-file-coding-system wl-cs-autoconv))
-		(setq status (pgg-decrypt-region beg end))
-		(pgg-display-output-buffer beg end status))
-	    (message "Cannot find pgp encrypted region")))
-      (message "Cannot find pgp encrypted region"))))
+    (let ((region (wl-find-region "^-+BEGIN PGP MESSAGE-+$"
+				  "^-+END PGP MESSAGE-+$"))
+	  (inhibit-read-only t)
+	  coding-system)
+      (unless region
+	(error "Cannot find pgp encrypted region"))
+      (save-restriction
+	(let ((props (text-properties-at (car region))))
+	  (narrow-to-region (car region) (cdr region))
+	  (when (setq coding-system
+		      (wl-mime-pgp-decrypt-region (point-min) (point-max)))
+	    (setq props (plist-put props
+				   'wl-mime-decoded-coding-system
+				   coding-system)))
+	  (set-text-properties (point-min) (point-max) props))))))
 
 (defun wl-message-verify-pgp-nonmime (&optional arg)
   "Verify PGP signed region.
 With ARG, ask coding system and encode the region with it before verifying."
   (interactive "P")
-  (require 'pgg)
   (save-excursion
     (beginning-of-line)
-    (let ((message-buffer (current-buffer))
-	  beg end coding-system success)
-      (setq end (and (or (re-search-forward "^-+END PGP SIGNATURE-+$" nil t)
-			 (re-search-backward "^-+END PGP SIGNATURE-+$" nil t)
-			 (error "Cannot find pgp signed region"))
-		     (match-end 0)))
-      (setq beg (or (re-search-backward "^-+BEGIN PGP SIGNED MESSAGE-+$" nil t)
-		    (error "Cannot find pgp signed region")))
+    (let ((region (wl-find-region "^-+BEGIN PGP SIGNED MESSAGE-+$"
+				  "^-+END PGP SIGNATURE-+$"))
+	  coding-system)
       (setq coding-system
-	    (or (let* ((situation (mime-preview-find-boundary-info))
+	    (or (get-text-property (car region) 'wl-mime-decoded-coding-system)
+		(let* ((situation (mime-preview-find-boundary-info))
 		       (entity (aref situation 2)))
 		  (mime-charset-to-coding-system
 		   (mime-content-type-parameter
@@ -522,21 +590,10 @@ With ARG, ask coding system and encode the region with it before verifying."
 	(setq coding-system (read-coding-system
 			     (format "Coding system (%S): " coding-system)
 			     coding-system)))
-      (with-temp-buffer
-	(insert-buffer-substring message-buffer beg end)
-	(encode-coding-region (point-min) (point-max) coding-system)
-	(setq success (pgg-verify-region (point-min) (point-max) nil 'fetch)))
-      (mime-show-echo-buffer)
-      (set-buffer mime-echo-buffer-name)
-      (set-window-start
-       (get-buffer-window mime-echo-buffer-name)
-       (point-max))
-      (insert-buffer-substring
-       (if success pgg-output-buffer pgg-errors-buffer)))))
+      (wl-mime-pgp-verify-region (car region) (cdr region) coding-system))))
 
 ;; XXX: encrypted multipart isn't represented as multipart
 (defun wl-mime-preview-application/pgp (parent-entity entity situation)
-  (require 'pgg)
   (goto-char (point-max))
   (let ((p (point))
 	raw-buf to-buf representation-type child-entity)
@@ -550,9 +607,7 @@ With ARG, ask coding system and encode the region with it before verifying."
 	(when (progn
 		(goto-char (point-min))
 		(re-search-forward "^-+BEGIN PGP MESSAGE-+$" nil t))
-	  (pgg-decrypt-region (point-min)(point-max))
-	  (delete-region (point-min) (point-max))
-	  (insert-buffer pgg-output-buffer)
+	  (wl-mime-pgp-decrypt-region (point-min) (point-max) 'no-decode)
 	  (setq representation-type 'elmo-buffer))
 	(setq child-entity (mime-parse-message
 			    (mm-expand-class-name representation-type)
@@ -594,6 +649,7 @@ With ARG, ask coding system and encode the region with it before verifying."
 	   (setq entity
 		 (car (mime-entity-children message-entity)))
 	   (with-temp-buffer
+	     (set-buffer-multibyte nil)
 	     (insert (mime-entity-body message-entity))
 	     (elmo-folder-append-buffer target))))
     number))
@@ -676,11 +732,11 @@ With ARG, ask destination folder."
 	(with-current-buffer mother
 	  (mime-store-message/partial-piece entity situation)
 	  (setq wl-message-buffer-cur-summary-buffer summary-buf)
-	  (make-variable-buffer-local 'mime-preview-over-to-next-method-alist)
+	  (make-local-variable 'mime-preview-over-to-next-method-alist)
 	  (setq mime-preview-over-to-next-method-alist
 		(cons (cons 'mime-show-message-mode 'wl-message-exit)
 		      mime-preview-over-to-next-method-alist))
-	  (make-variable-buffer-local 'mime-preview-over-to-previous-method-alist)
+	  (make-local-variable 'mime-preview-over-to-previous-method-alist)
 	  (setq mime-preview-over-to-previous-method-alist
 		(cons (cons 'mime-show-message-mode 'wl-message-exit)
 		      mime-preview-over-to-previous-method-alist)))
