@@ -293,6 +293,7 @@ ENTITY is returned."
       (cdr (memq (car entity) wl-thread-entity-list)))))
 
 (defun wl-thread-jump-to-msg (&optional number)
+  "Jump to the message with specified number in the current summary."
   (interactive)
   (let ((num (or number
 		 (string-to-int
@@ -385,10 +386,7 @@ ENTITY is returned."
 	      (elmo-message-entity wl-summary-buffer-elmo-folder
 				   parent-msg)
 	      temp-mark
-	      (elmo-message-flags wl-summary-buffer-elmo-folder
-				  msg)
-	      (elmo-message-cached-p wl-summary-buffer-elmo-folder
-				     msg)
+	      (elmo-message-status wl-summary-buffer-elmo-folder msg)
 	      (if wl-thread-insert-force-opened
 		  nil
 		(wl-thread-maybe-get-children-num msg))
@@ -765,6 +763,11 @@ Message is inserted to the summary buffer."
   (interactive "P")
   (wl-thread-call-region-func 'wl-summary-mark-as-answered-region arg))
 
+(defun wl-thread-recover-messages (&optional arg)
+  "Recover killed messages which are contained current thread."
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-recover-messages-region arg))
+
 (defun wl-thread-unmark (&optional arg)
   (interactive "P")
   (wl-thread-call-region-func 'wl-summary-unmark-region arg))
@@ -779,10 +782,9 @@ Message is inserted to the summary buffer."
 
 (defun wl-thread-force-open (&optional msg-num)
   "force open current folder"
-  (if msg-num
-      (wl-summary-jump-to-msg msg-num))
-  (let ((wl-thread-insert-force-opened t))
-    (wl-thread-open-close)))
+  (when msg-num
+    (wl-summary-jump-to-msg msg-num))
+  (wl-thread-open-close 'force-open))
 
 (defun wl-thread-entity-force-open (entity)
   (let ((wl-thread-insert-force-opened t)
@@ -838,10 +840,7 @@ Message is inserted to the summary buffer."
 	  (elmo-message-entity wl-summary-buffer-elmo-folder
 			       (nth 0 parent-entity))
 	  temp-mark
-	  (elmo-message-flags wl-summary-buffer-elmo-folder
-			      msg-num)
-	  (elmo-message-cached-p wl-summary-buffer-elmo-folder
-				 msg-num)
+	  (elmo-message-status wl-summary-buffer-elmo-folder msg-num)
 	  (if wl-thread-insert-force-opened
 	      nil
 	    (wl-thread-maybe-get-children-num msg-num))
