@@ -336,6 +336,14 @@ changing the value of `foo'."
     (setq keys (cdr keys)))
   alist)
 
+(defun wl-filter-associations (keys alist)
+  (let (entry result)
+    (while keys
+      (when (setq entry (assq (car keys) alist))
+	(setq result (cons entry result)))
+      (setq keys (cdr keys)))
+    result))
+
 (defun wl-inverse-alist (keys alist)
   "Inverse ALIST, copying.
 Return an association list represents the inverse mapping of ALIST,
@@ -517,28 +525,35 @@ that `read' can handle, whenever this is possible."
     result))
 
 (defun wl-collect-draft ()
-  (let ((draft-regexp (concat
-		       "^" (regexp-quote wl-draft-folder)))
-	result buf)
-    (mapcar
-     (function (lambda (x)
-		 (if (with-current-buffer x
-		       (and (eq major-mode 'wl-draft-mode)
-			    (buffer-name)
-			    (string-match draft-regexp (buffer-name))))
-		     (setq result (nconc result (list x))))))
-     (buffer-list))
-    result))
+  (let ((draft-regexp (concat "^" (regexp-quote wl-draft-folder)))
+	result)
+    (dolist (buffer (buffer-list))
+      (when (with-current-buffer buffer
+	      (and (eq major-mode 'wl-draft-mode)
+		   (buffer-name)
+		   (string-match draft-regexp (buffer-name))))
+	(setq result (cons buffer result))))
+    (nreverse result)))
+
+(defvar wl-inhibit-save-drafts nil)
+(defvar wl-disable-auto-save nil)
+(make-variable-buffer-local 'wl-disable-auto-save)
 
 (defun wl-save-drafts ()
-  (let ((msg (current-message))
-	(buffers (wl-collect-draft)))
-    (save-excursion
-      (while buffers
-	(set-buffer (car buffers))
-	(if (buffer-modified-p) (wl-draft-save))
-	(setq buffers (cdr buffers))))
-    (message "%s" (or msg ""))))
+  "Save all drafts. Return nil if there is no draft buffer."
+  (if wl-inhibit-save-drafts
+      'inhibited
+    (let ((wl-inhibit-save-drafts t)
+	  (msg (current-message))
+	  (buffers (wl-collect-draft)))
+      (save-excursion
+	(dolist (buffer buffers)
+	  (set-buffer buffer)
+	  (when (and (not wl-disable-auto-save)
+		     (buffer-modified-p))
+	    (wl-draft-save))))
+      (message "%s" (or msg ""))
+      buffers)))
 
 (static-if (fboundp 'read-directory-name)
     (defun wl-read-directory-name (prompt dir)
@@ -670,32 +685,54 @@ that `read' can handle, whenever this is possible."
 	max))))
 
 ;; Draft auto-save
+(defun wl-auto-save-drafts ()
+  (unless (wl-save-drafts)
+    (wl-stop-save-drafts)))
+
 (static-cond
  (wl-on-xemacs
   (defvar wl-save-drafts-timer-name "wl-save-drafts")
 
-  (defun wl-set-save-drafts ()
-    (if (numberp wl-auto-save-drafts-interval)
-	(unless (get-itimer wl-save-drafts-timer-name)
-	  (start-itimer wl-save-drafts-timer-name 'wl-save-drafts
-			wl-auto-save-drafts-interval wl-auto-save-drafts-interval
-			t))
-      (when (get-itimer wl-save-drafts-timer-name)
-	(delete-itimer wl-save-drafts-timer-name)))))
+  (defun wl-start-save-drafts ()
+    (when (numberp wl-auto-save-drafts-interval)
+      (unless (get-itimer wl-save-drafts-timer-name)
+	(start-itimer wl-save-drafts-timer-name
+		      'wl-auto-save-drafts
+		      wl-auto-save-drafts-interval
+		      wl-auto-save-drafts-interval
+		      t))))
+
+  (defun wl-stop-save-drafts ()
+    (when (get-itimer wl-save-drafts-timer-name)
+      (delete-itimer wl-save-drafts-timer-name))))
  (t
-  (defun wl-set-save-drafts ()
-    (if (numberp wl-auto-save-drafts-interval)
-	(progn
-	  (require 'timer)
-	  (if (get 'wl-save-drafts 'timer)
-	      (progn (timer-set-idle-time (get 'wl-save-drafts 'timer)
-					  wl-auto-save-drafts-interval t)
-		     (timer-activate-when-idle (get 'wl-save-drafts 'timer)))
-	    (put 'wl-save-drafts 'timer
-		 (run-with-idle-timer
-		  wl-auto-save-drafts-interval t 'wl-save-drafts))))
-      (when (get 'wl-save-drafts 'timer)
-	(cancel-timer (get 'wl-save-drafts 'timer)))))))
+  (defun wl-start-save-drafts ()
+    (when (numberp wl-auto-save-drafts-interval)
+      (require 'timer)
+      (if (get 'wl-save-drafts 'timer)
+	  (progn
+	    (timer-set-idle-time (get 'wl-save-drafts 'timer)
+				 wl-auto-save-drafts-interval t)
+	    (timer-activate-when-idle (get 'wl-save-drafts 'timer)))
+	(put 'wl-save-drafts 'timer
+	     (run-with-idle-timer
+	      wl-auto-save-drafts-interval t 'wl-auto-save-drafts)))))
+
+  (defun wl-stop-save-drafts ()
+    (when (get 'wl-save-drafts 'timer)
+      (cancel-timer (get 'wl-save-drafts 'timer))))))
+
+(defun wl-set-auto-save-draft (&optional arg)
+  (interactive "P")
+  (unless (setq wl-disable-auto-save
+		(cond
+		 ((null arg) (not wl-disable-auto-save))
+		 ((< (prefix-numeric-value arg) 0) t)
+		 (t nil)))
+    (wl-start-save-drafts))
+  (when (interactive-p)
+    (message "Auto save is %s (in this buffer)"
+	     (if wl-disable-auto-save "disabled" "enabled"))))
 
 ;; Biff
 (static-cond
@@ -996,22 +1033,28 @@ is enclosed by at least one regexp grouping construct."
 	     (symbol-value variable))))))
 
 ;;; Search Condition
+(defun wl-search-condition-fields ()
+  (let ((denial-fields
+	 (nconc (mapcar 'capitalize elmo-msgdb-extra-fields)
+		(mapcar 'capitalize wl-additional-search-condition-fields)
+		'("Flag" "Since" "Before"
+		  "From" "Subject" "To" "Cc" "Body" "ToCc"
+		  "Larger" "Smaller"))))
+    (append '("Last" "First")
+	    denial-fields
+	    (mapcar (lambda (f) (concat "!" f))
+		    denial-fields))))
+
 (defun wl-read-search-condition (default)
   "Read search condition string interactively."
   (wl-read-search-condition-internal "Search by" default))
 
 (defun wl-read-search-condition-internal (prompt default &optional paren)
   (let* ((completion-ignore-case t)
-	 (denial-fields (nconc (mapcar 'capitalize elmo-msgdb-extra-fields)
-			       '("Flag" "Since" "Before"
-				 "From" "Subject" "To" "Cc" "Body" "ToCc")))
 	 (field (completing-read
 		 (format "%s (%s): " prompt default)
-		 (mapcar 'list
-			 (append '("AND" "OR" "Last" "First")
-				 denial-fields
-				 (mapcar (lambda (f) (concat "!" f))
-					 denial-fields)))))
+		 (mapcar #'list
+			 (append '("AND" "OR") (wl-search-condition-fields)))))
 	 value)
     (setq field (if (string= field "")
 		    (setq field default)
@@ -1061,6 +1104,85 @@ is enclosed by at least one regexp grouping construct."
 	(setq value (prin1-to-string value)))
       (concat (downcase field) ":" value)))))
 
+(defun wl-y-or-n-p-with-scroll (prompt &optional scroll-by-SPC)
+  (let ((prompt (concat prompt (if scroll-by-SPC
+				   "<y/n/SPC(down)/BS(up)> "
+				 "<y/n/j(down)/k(up)> "))))
+    (catch 'done
+      (while t
+	(discard-input)
+	(case (let ((cursor-in-echo-area t))
+		(cdr (wl-read-event-char prompt)))
+	  ((?y ?Y)
+	   (throw 'done t))
+	  (?\ 
+	   (if scroll-by-SPC
+	       (ignore-errors (scroll-up))
+	     (throw 'done t)))
+	  ((?v ?j ?J next)
+	   (ignore-errors (scroll-up)))
+	  ((?^ ?k ?K prior backspace)
+	   (ignore-errors (scroll-down)))
+	  (t
+	   (throw 'done nil)))))))
+
+(defun wl-find-region (beg-regexp end-regexp)
+  (if (or (re-search-forward end-regexp nil t)
+	  (re-search-backward end-regexp nil t))
+      (let ((end (match-end 0))
+	    (beg (re-search-backward beg-regexp nil t)))
+	(if beg
+	    (cons beg end)))))
+
+;; read multiple strings with completion
+(defun wl-completing-read-multiple-1 (prompt
+				      table
+				      &optional predicate
+				      require-match initial-input
+				      hist def inherit-input-method)
+    "Read multiple strings in the minibuffer"
+    (split-string
+     (completing-read prompt table predicate nil
+		      initial-input hist def inherit-input-method)
+     ","))
+
+(static-when (fboundp 'completing-read-multiple)
+  (eval-when-compile
+    (require 'crm))
+  (defun wl-completing-read-multiple-2 (prompt
+					table
+					&optional predicate
+					require-match initial-input
+					hist def inherit-input-method)
+    "Read multiple strings in the minibuffer"
+    (let ((ret (completing-read-multiple prompt table predicate
+					 require-match initial-input
+					 hist def inherit-input-method)))
+      (if (and def (equal ret '("")))
+	  (split-string def crm-separator)
+	ret))))
+
+(static-cond
+ ((not (fboundp 'completing-read-multiple))
+  (defalias 'wl-completing-read-multiple 'wl-completing-read-multiple-1))
+ ((< emacs-major-version 22)
+  (defun wl-completing-read-multiple (prompt
+				      table
+				      &optional predicate
+				      require-match initial-input
+				      hist def inherit-input-method)
+    "Read multiple strings in the minibuffer"
+    (if require-match
+	(wl-completing-read-multiple-1 prompt table predicate
+				       nil initial-input
+				       hist def inherit-input-method)
+      (wl-completing-read-multiple-2 prompt table predicate
+				     nil initial-input
+				     hist def inherit-input-method))))
+ (t
+  (defalias 'wl-completing-read-multiple 'wl-completing-read-multiple-2)))
+
+
 (require 'product)
 (product-provide (provide 'wl-util) (require 'wl-version))
 
