@@ -137,7 +137,8 @@
 (defvar wl-plugged-port-label-alist
   (list (cons 119 "nntp")
 	(cons 143 "imap4")
-	(cons 110 "pop3")))
+	(cons 110 "pop3")
+	(cons 25 "smtp")))
 	;;(cons elmo-pop-before-smtp-port "pop3")
 
 (defconst wl-plugged-switch-variables
@@ -673,7 +674,6 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
     (when wl-use-acap (funcall (symbol-function 'wl-acap-exit)))
     (wl-biff-stop)
     (elmo-clear-signal-slots)
-    (elmo-passwd-alist-clear)
     (run-hooks 'wl-exit-hook)
     (wl-save-status)
     (wl-folder-cleanup-variables)
@@ -691,6 +691,7 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 ;;	(delete-frame))
     (setq wl-init nil)
     (remove-hook 'kill-emacs-hook 'wl-save-status)
+    (elmo-passwd-alist-clear)
     t)
   (message "") ; empty minibuffer.
   )
@@ -712,6 +713,7 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	  (symbol-value 'wl-summary-subject-function))
     (fset 'wl-summary-subject-filter-func-internal
 	  (symbol-value 'wl-summary-subject-filter-function))
+    (wl-summary-define-sort-command)
     (wl-summary-define-mark-action)
     (dolist (spec wl-summary-flag-alist)
       (set-face-foreground
